﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
namespace WawaSoft.Common
{
    public class PerfCounterFactory
    {
        static Dictionary<string, PerfCounter> _counters = new Dictionary<string, PerfCounter>();
        static object _countsLocker = new object();
        public static T GetCounters<T>()
            where T : IPerfCounterProvider, new()
        {
            T t = new T();
            t.CreateCounters();
            CreateCounters(t.PerformanceObjectName(), true);
            t.CountersCreated();
            return t;
        }

        public static PerformanceCounter Find(string perfCountName)
        {
            lock (_countsLocker)
            {
                PerfCounter counter;
                _counters.TryGetValue(perfCountName, out counter);
                return counter.Counter;
            }
        }
        public static void AddPerfCounter(PerfCounter counter)
        {
            lock (_countsLocker)
            {
                _counters.Add(counter.Name, counter);
            }
        }

        static void CreateCounters(string PerformanceObjectName, bool force)
        {
            // does the object exist?
            bool countersExist = PerformanceCounterCategory.Exists(PerformanceObjectName);

            // delete the category?
            if (countersExist == true && force == true)
            {
                PerformanceCounterCategory.Delete(PerformanceObjectName);
                countersExist = false;
            }

            // do we need to create it?
            if (countersExist == false)
            {
                // create a collection...
                CounterCreationDataCollection list = new CounterCreationDataCollection();

                lock (_countsLocker)
                {
                    // go through each counter...
                    foreach (PerfCounter counter in _counters.Values)
                    {
                        // create some new data...
                        CounterCreationData data = new CounterCreationData();
                        data.CounterName = counter.Name;
                        data.CounterHelp = counter.HelpText;
                        data.CounterType = counter.Type;

                        // add it...
                        list.Add(data);
                    }

                }
                // create the category and all of the counters...
                PerformanceCounterCategory.Create(PerformanceObjectName, "", PerformanceCounterCategoryType.SingleInstance, list);
            }

            lock (_countsLocker)
            {
                // now, go back through the counters and create instances...
                foreach (PerfCounter counter in _counters.Values)
                {
                    // create an instance and store it...
                    counter.Counter = new PerformanceCounter(PerformanceObjectName, counter.Name, "", false);

                    // reset the value...
                    counter.Counter.RawValue = 0;
                }
            }


        }
    }
}
